#include <J3ML/LinearAlgebra/EulerAngle.h>
#include <cmath>
#include <algorithm>


#pragma region EulerAngle
namespace LinearAlgebra {
EulerAngle::EulerAngle(float pitch, float yaw, float roll): pitch(pitch), yaw(yaw), roll(roll)
{}

float EulerAngle::GetPitch(float pitch_limit) const
{ return std::clamp( std::remainderf(pitch,360.f), -pitch_limit, pitch_limit); }

float EulerAngle::GetYaw(float yaw_limit) const
{ return std::clamp(std::remainderf(yaw, 360.f), -yaw_limit, yaw_limit);   }

float EulerAngle::GetRoll(float pitch_limit) const
{ return std::clamp( std::remainderf(pitch,360.f), -pitch_limit, pitch_limit); }

bool EulerAngle::operator==(const EulerAngle& a) const
{
    return (pitch == a.pitch) && (yaw == a.yaw) && (roll == a.roll);
}

void EulerAngle::clamp()
{
    if (this->pitch > 89.0f)
        this->pitch = 89.0f;
    if (this->pitch <= -89.0f)
        this->pitch = -89.0f;
    //TODO: Make this entirely seamless by getting the amount they rotated passed -180 and +180 by.
    if (this->yaw <= -180.0f)
        this->yaw = 180.0f;
    if (this->yaw >= 180.01f)
        this->yaw = -179.9f;
    if (this->roll >= 360.0f)
        this->roll = 0.0;
    if (this->roll <= -360.0f)
        this->roll = 0.0;
}

EulerAngle EulerAngle::movementAngle() const
{
    EulerAngle a;
    a.pitch = (cos(Math::Radians(yaw)) * cos(Math::Radians(pitch)));
    a.yaw = -sin(Math::Radians(pitch));
    a.roll = (sin(Math::Radians(yaw)) * cos(Math::Radians(pitch)));
    return a;
}
}